package java.rmi.server;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.util.ServiceLoader;
import sun.rmi.server.LoaderHandler;
import java.net.URL;
import java.net.MalformedURLException;

public class RMIClassLoader
{
    private static final RMIClassLoaderSpi defaultProvider;
    private static final RMIClassLoaderSpi provider;
    
    private RMIClassLoader() {
    }
    
    @Deprecated
    public static Class<?> loadClass(String s) throws MalformedURLException, ClassNotFoundException {
        return loadClass((String)null, s);
    }
    
    public static Class<?> loadClass(URL url, String s) throws MalformedURLException, ClassNotFoundException {
        return RMIClassLoader.provider.loadClass((url != null) ? url.toString() : null, s, null);
    }
    
    public static Class<?> loadClass(String s, String s2) throws MalformedURLException, ClassNotFoundException {
        return RMIClassLoader.provider.loadClass(s, s2, null);
    }
    
    public static Class<?> loadClass(String s, String s2, ClassLoader classLoader) throws MalformedURLException, ClassNotFoundException {
        return RMIClassLoader.provider.loadClass(s, s2, classLoader);
    }
    
    public static Class<?> loadProxyClass(String s, String[] array, ClassLoader classLoader) throws ClassNotFoundException, MalformedURLException {
        return RMIClassLoader.provider.loadProxyClass(s, array, classLoader);
    }
    
    public static ClassLoader getClassLoader(String s) throws MalformedURLException, SecurityException {
        return RMIClassLoader.provider.getClassLoader(s);
    }
    
    public static String getClassAnnotation(Class<?> clazz) {
        return RMIClassLoader.provider.getClassAnnotation(clazz);
    }
    
    public static RMIClassLoaderSpi getDefaultProviderInstance() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("setFactory"));
        }
        return RMIClassLoader.defaultProvider;
    }
    
    @Deprecated
    public static Object getSecurityContext(ClassLoader loader) {
        return LoaderHandler.getSecurityContext(loader);
    }
    
    private static RMIClassLoaderSpi newDefaultProviderInstance() {
        return new RMIClassLoaderSpi() {
            @Override
            public Class<?> loadClass(String codebase, String name, ClassLoader defaultLoader) throws MalformedURLException, ClassNotFoundException {
                return LoaderHandler.loadClass(codebase, name, defaultLoader);
            }
            
            @Override
            public Class<?> loadProxyClass(String codebase, String[] interfaces, ClassLoader defaultLoader) throws MalformedURLException, ClassNotFoundException {
                return LoaderHandler.loadProxyClass(codebase, interfaces, defaultLoader);
            }
            
            @Override
            public ClassLoader getClassLoader(String codebase) throws MalformedURLException {
                return LoaderHandler.getClassLoader(codebase);
            }
            
            @Override
            public String getClassAnnotation(Class<?> cl) {
                return LoaderHandler.getClassAnnotation(cl);
            }
        };
    }
    
    private static RMIClassLoaderSpi initializeProvider() {
        String property = System.getProperty("java.rmi.server.RMIClassLoaderSpi");
        if (property != null) {
            if (property.equals("default")) {
                return RMIClassLoader.defaultProvider;
            }
            try {
                return (RMIClassLoaderSpi)Class.forName(property, false, ClassLoader.getSystemClassLoader()).asSubclass(RMIClassLoaderSpi.class).newInstance();
            }
            catch (ClassNotFoundException ex) {
                throw new NoClassDefFoundError(ex.getMessage());
            }
            catch (IllegalAccessException ex2) {
                throw new IllegalAccessError(ex2.getMessage());
            }
            catch (InstantiationException ex3) {
                throw new InstantiationError(ex3.getMessage());
            }
            catch (ClassCastException cause2) {
                LinkageError linkageError2 = new LinkageError("provider class not assignable to RMIClassLoaderSpi");
                linkageError2.initCause(cause2);
                throw linkageError2;
            }
        }
        else {
            Iterator<RMIClassLoaderSpi> iterator = ServiceLoader.load(RMIClassLoaderSpi.class, ClassLoader.getSystemClassLoader()).iterator();
            if (iterator.hasNext()) {
                try {
                    return (RMIClassLoaderSpi)iterator.next();
                }
                catch (ClassCastException cause) {
                    LinkageError linkageError = new LinkageError("provider class not assignable to RMIClassLoaderSpi");
                    linkageError.initCause(cause);
                    throw linkageError;
                }
            }
            else {
                return RMIClassLoader.defaultProvider;
            }
        }
    }
    
    static RMIClassLoaderSpi access$000() {
        return initializeProvider();
    }
    
    static {
        defaultProvider = newDefaultProviderInstance();
        provider = (RMIClassLoaderSpi)AccessController.doPrivileged(new PrivilegedAction<RMIClassLoaderSpi>() {
            @Override
            public RMIClassLoaderSpi run() {
                return RMIClassLoader.access$000();
            }
        });
    }
}
